@@ -46,6 +46,12 @@ sip_connection_sofia_new (SIPConnection *conn)
   return sofia;
 }
 
+void
+sip_connection_sofia_destroy (SIPConnectionSofia *sofia)
+{
+  g_slice_free (SIPConnectionSofia, sofia);
+}
+
 static void
 priv_r_shutdown(int status,
                 char const *phrase, 
@@ -81,7 +87,7 @@ priv_r_shutdown(int status,
   if (!source_recursive)
     g_source_set_can_recurse (source, FALSE);
 
-  g_slice_free (SIPConnectionSofia, sofia);
+  sip_connection_sofia_destroy (sofia);
 }
 
 static void
@@ -749,10 +755,16 @@ priv_i_state (int status,
   if (nh_magic == SIP_NH_EXPIRED)
     {
       g_message ("call state change %03d '%s' received for a "
-          "destroyed media channel (handle %p), BYE off", status, phrase, nh);
-      if (ss_state != nua_callstate_terminated)
-        nua_bye (nh, TAG_END());
-      /* Destroy it? */
+          "destroyed media channel (handle %p)", status, phrase, nh);
+      switch (ss_state)
+        {
+        case nua_callstate_terminated:
+        case nua_callstate_terminating:
+          break;
+        default:
+          DEBUG("BYE off the orphaned handle");
+          nua_bye (nh, TAG_END());
+        }
       return;
     }
 
@@ -781,6 +793,10 @@ priv_i_state (int status,
     break;
 
   case nua_callstate_terminated:
+    {
+      SIPConnectionPrivate *priv = SIP_CONNECTION_GET_PRIVATE (self);
+      priv_auth_drop (priv->auth_table, nh);
+    }
     sip_media_channel_terminated (channel);
     break;
 
