@@ -407,11 +407,11 @@ sip_media_channel_dispose (GObject *object)
   SIPMediaChannel *self = SIP_MEDIA_CHANNEL (object);
   SIPMediaChannelPrivate *priv = SIP_MEDIA_CHANNEL_GET_PRIVATE (self);
 
-  DEBUG("enter");
-
   if (priv->dispose_has_run)
     return;
 
+  DEBUG("enter");
+
   priv->dispose_has_run = TRUE;
 
   priv_destroy_session(self);
@@ -428,7 +428,7 @@ sip_media_channel_dispose (GObject *object)
   if (G_OBJECT_CLASS (sip_media_channel_parent_class)->dispose)
     G_OBJECT_CLASS (sip_media_channel_parent_class)->dispose (object);
 
-  DEBUG ("exit");
+  DEBUG("exit");
 }
 
 void
@@ -445,7 +445,7 @@ sip_media_channel_finalize (GObject *object)
 
   G_OBJECT_CLASS (sip_media_channel_parent_class)->finalize (object);
 
-  DEBUG ("exit");
+  DEBUG("exit");
 }
 
 /***********************************************************************
@@ -938,6 +938,9 @@ sip_media_channel_peer_error (SIPMediaChannel *self,
 
   peer = sip_media_session_get_peer (priv->session);
 
+  if (message == NULL || !g_utf8_validate (message, -1, NULL))
+    message = "";
+
   set = tp_intset_new ();
   tp_intset_add (set, peer);
   tp_group_mixin_change_members ((GObject *)self, message,
@@ -948,7 +951,7 @@ sip_media_channel_peer_error (SIPMediaChannel *self,
 void
 sip_media_channel_peer_cancel (SIPMediaChannel *self,
                                guint cause,
-                               const gchar *text)
+                               const gchar *message)
 {
   SIPMediaChannelPrivate *priv = SIP_MEDIA_CHANNEL_GET_PRIVATE (self);
   TpGroupMixin *mixin = TP_GROUP_MIXIN (self);
@@ -971,15 +974,15 @@ sip_media_channel_peer_cancel (SIPMediaChannel *self,
       actor = peer;
     }
 
-  if (text == NULL)
-    text = "Cancelled";
+  if (message == NULL || !g_utf8_validate (message, -1, NULL))
+    message = "Cancelled";
 
   set = tp_intset_new ();
   tp_intset_add (set, peer);
   tp_intset_add (set, mixin->self_handle);
 
   tp_group_mixin_change_members ((GObject *) self,
-                                 text,
+                                 message,
                                  NULL, /* add */
                                  set,  /* remove */
                                  NULL,
@@ -1081,6 +1084,7 @@ priv_create_session (SIPMediaChannel *channel,
   TpBaseConnection *conn;
   TpHandleRepoIface *contact_repo;
   gchar *object_path;
+  gchar *local_ip_address = NULL;
 
   DEBUG("enter");
 
@@ -1097,13 +1101,20 @@ priv_create_session (SIPMediaChannel *channel,
   /* The channel manages references to the peer handle for the session */
   tp_handle_ref (contact_repo, peer);
 
+  g_object_get (priv->conn,
+                "local-ip-address", &local_ip_address,
+                NULL);
+
   session = g_object_new (SIP_TYPE_MEDIA_SESSION,
                           "media-channel", channel,
                           "object-path", object_path,
                           "nua-handle", nh,
                           "peer", peer,
+                          "local-ip-address", local_ip_address,
                           NULL);
 
+  g_free (local_ip_address);
+
   g_signal_connect (session, "notify::state",
                     (GCallback) priv_session_state_changed_cb, channel);
   g_signal_connect (session, "stream-added",
@@ -1130,12 +1141,12 @@ priv_destroy_session(SIPMediaChannel *channel)
   TpBaseConnection *conn;
   TpHandleRepoIface *contact_repo;
 
-  DEBUG("enter");
-
   session = priv->session;
   if (session == NULL)
     return;
 
+  DEBUG("enter");
+
   /* Release the peer handle */
   conn = (TpBaseConnection *)(priv->conn);
   contact_repo = tp_base_connection_get_handles (conn,
@@ -1144,6 +1155,8 @@ priv_destroy_session(SIPMediaChannel *channel)
 
   priv->session = NULL;
   g_object_unref (session);
+
+  DEBUG("exit");
 }
 
 #if 0
