@@ -70,6 +70,7 @@ enum
   PROP_OBJECT_PATH,
   PROP_NUA_OP,
   PROP_PEER,
+  PROP_LOCAL_IP_ADDRESS,
   PROP_STATE,
   LAST_PROPERTY
 };
@@ -83,6 +84,7 @@ static guint signals[SIG_LAST_SIGNAL] = {0};
  * - created, objects created, local cand/codec query ongoing
  * - invite-sent, an INVITE with local SDP sent, awaiting response
  * - invite-received, a remote INVITE received, response is pending
+ * - response-received, a 200 OK received, codec intersection is in progress
  * - active, codecs and candidate pairs have been negotiated (note,
  *   SteamEngine might still fail to verify connectivity and report
  *   an error)
@@ -95,6 +97,7 @@ static const char* session_states[] =
     "created",
     "invite-sent",
     "invite-received",
+    "response-received",
     "active",
     "reinvite-sent",
     "reinvite-received",
@@ -112,6 +115,7 @@ struct _SIPMediaSessionPrivate
   gchar *object_path;                   /** see gobj. prop. 'object-path' */
   nua_handle_t *nua_op;                 /** see gobj. prop. 'nua-handle' */
   TpHandle peer;                        /** see gobj. prop. 'peer' */
+  gchar *local_ip_address;              /** see gobj. prop. 'local-ip-address' */
   SIPMediaSessionState state;           /** see gobj. prop. 'state' */
   nua_saved_event_t saved_event[1];     /** Saved incoming request event */
   gint local_non_ready;                 /** number of streams with local information update pending */
@@ -223,6 +227,9 @@ static void sip_media_session_get_property (GObject    *object,
     case PROP_PEER:
       g_value_set_uint (value, priv->peer);
       break;
+    case PROP_LOCAL_IP_ADDRESS:
+      g_value_set_string (value, priv->local_ip_address);
+      break;
     case PROP_STATE:
       g_value_set_uint (value, priv->state);
       break;
@@ -242,10 +249,10 @@ static void sip_media_session_set_property (GObject      *object,
 
   switch (property_id) {
     case PROP_MEDIA_CHANNEL:
-      priv->channel = SIP_MEDIA_CHANNEL (g_value_dup_object (value));
+      priv->channel = SIP_MEDIA_CHANNEL (g_value_get_object (value));
       break;
     case PROP_OBJECT_PATH:
-      g_free (priv->object_path);
+      g_assert (priv->object_path == NULL);
       priv->object_path = g_value_dup_string (value);
       break;
     case PROP_NUA_OP:
@@ -258,6 +265,10 @@ static void sip_media_session_set_property (GObject      *object,
     case PROP_PEER:
       priv->peer = g_value_get_uint (value);
       break;
+    case PROP_LOCAL_IP_ADDRESS:
+      g_assert (priv->local_ip_address == NULL);
+      priv->local_ip_address = g_value_dup_string (value);
+      break;
     case PROP_STATE:
       priv_session_state_changed (session, g_value_get_uint (value));
       break;
@@ -288,7 +299,7 @@ sip_media_session_class_init (SIPMediaSessionClass *sip_media_session_class)
 
   param_spec = g_param_spec_object ("media-channel", "SIPMediaChannel object",
                                     "SIP media channel object that owns this "
-                                    "media session object.",
+                                    "media session object (not reference counted).",
                                     SIP_TYPE_MEDIA_CHANNEL,
                                     G_PARAM_CONSTRUCT_ONLY |
                                     G_PARAM_READWRITE |
@@ -325,6 +336,16 @@ sip_media_session_class_init (SIPMediaSessionClass *sip_media_session_class)
                                   G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_PEER, param_spec);
 
+  param_spec = g_param_spec_string ("local-ip-address", "Local IP address",
+                                    "The local IP address preferred for "
+                                    "media streams",
+                                    NULL,
+                                    G_PARAM_CONSTRUCT_ONLY |
+                                    G_PARAM_READWRITE |
+                                    G_PARAM_STATIC_NAME |
+                                    G_PARAM_STATIC_BLURB);
+  g_object_class_install_property (object_class, PROP_LOCAL_IP_ADDRESS, param_spec);
+
   param_spec = g_param_spec_uint ("state", "Session state",
                                   "The current state that the session is in.",
                                   0, G_MAXUINT32, 0,
@@ -352,6 +373,8 @@ sip_media_session_dispose (GObject *object)
   if (priv->dispose_has_run)
     return;
 
+  DEBUG("enter");
+
   priv->dispose_has_run = TRUE;
 
   if (priv->catcher_id)
@@ -360,31 +383,24 @@ sip_media_session_dispose (GObject *object)
   if (priv->timer_id)
     g_source_remove (priv->timer_id);
 
-  if (priv->channel)
-    g_object_unref (priv->channel);
-
   if (G_OBJECT_CLASS (sip_media_session_parent_class)->dispose)
     G_OBJECT_CLASS (sip_media_session_parent_class)->dispose (object);
 
-  DEBUG ("exit");
+  DEBUG("exit");
 }
 
-void
+static void
 sip_media_session_finalize (GObject *object)
 {
   SIPMediaSession *self = SIP_MEDIA_SESSION (object);
   SIPMediaSessionPrivate *priv = SIP_MEDIA_SESSION_GET_PRIVATE (self);
   guint i;
 
-  DEBUG ("enter");
-
-  /* free any data held directly by the object here */
-
-  G_OBJECT_CLASS (sip_media_session_parent_class)->finalize (object);
-
   /* terminating the session should have discarded the NUA handle */
   g_assert (priv->nua_op == NULL);
 
+  /* free any data held directly by the object here */
+
   for (i = 0; i < priv->streams->len; i++) {
     SIPMediaStream *stream = g_ptr_array_index (priv->streams, i);
     if (stream != NULL)
@@ -402,7 +418,12 @@ sip_media_session_finalize (GObject *object)
   if (priv->backup_home != NULL)
     su_home_unref (priv->backup_home);
 
-  DEBUG ("exit");
+  g_free (priv->local_ip_address);
+  g_free (priv->object_path);
+
+  G_OBJECT_CLASS (sip_media_session_parent_class)->finalize (object);
+
+  DEBUG("exit");
 }
 
 
@@ -576,6 +597,8 @@ priv_session_state_changed (SIPMediaSession *session,
       priv->timer_id =
         g_timeout_add (DEFAULT_SESSION_TIMEOUT, priv_timeout_session, session);
       break;
+    case SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED:
+      break;
     case SIP_MEDIA_SESSION_STATE_ACTIVE:
       if (priv->timer_id)
         {
@@ -687,6 +710,7 @@ void sip_media_session_terminate (SIPMediaSession *session)
   if (priv->state == SIP_MEDIA_SESSION_STATE_ENDED)
     return;
 
+  /* XXX: taken care of by the state change? */
   priv_close_all_streams (session);
 
   if (priv->nua_op != NULL)
@@ -696,6 +720,7 @@ void sip_media_session_terminate (SIPMediaSession *session)
       switch (priv->state)
         {
         case SIP_MEDIA_SESSION_STATE_ACTIVE:
+        case SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED:
         case SIP_MEDIA_SESSION_STATE_REINVITE_SENT:
           DEBUG("sending BYE");
           nua_bye (priv->nua_op, TAG_END());
@@ -746,6 +771,13 @@ sip_media_session_set_remote_media (SIPMediaSession *session,
       priv->catcher_id = 0;
     }
 
+  /* Switch the state machine to processing the response */
+  if (priv->state == SIP_MEDIA_SESSION_STATE_INVITE_SENT
+      || priv->state == SIP_MEDIA_SESSION_STATE_REINVITE_SENT)
+    g_object_set (session,
+                  "state", SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED,
+                  NULL);
+
   /* Handle session non-updates */
   if (!sdp_session_cmp (priv->remote_sdp, sdp))
     {
@@ -1004,15 +1036,8 @@ void
 sip_media_session_receive_reinvite (SIPMediaSession *self)
 {
   SIPMediaSessionPrivate *priv = SIP_MEDIA_SESSION_GET_PRIVATE (self);
-
-  /* Note: the Sofia-SIP stack is supposed to weed out wrongly sequenced
-   * INVITE requests with session offers; here we may have a case
-   * when the processing of the response did not complete before we
-   * return back to active */
-
   g_return_if_fail (priv->state == SIP_MEDIA_SESSION_STATE_ACTIVE
-                    || priv->state == SIP_MEDIA_SESSION_STATE_INVITE_SENT
-                    || priv->state == SIP_MEDIA_SESSION_STATE_REINVITE_SENT);
+                    || priv->state == SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED);
 
   priv_save_event (self);
 
@@ -1132,9 +1157,33 @@ sip_media_session_stop_telephony_event  (SIPMediaSession *self,
   return TRUE;
 }
 
-/***********************************************************************
- * Helper functions follow (not based on generated templates)
- ***********************************************************************/
+gint
+sip_media_session_rate_native_transport (SIPMediaSession *session,
+                                         const GValue *transport)
+{
+  SIPMediaSessionPrivate *priv = SIP_MEDIA_SESSION_GET_PRIVATE (session);
+  gint result = 0;
+  gchar *address = NULL;
+  guint proto = TP_MEDIA_STREAM_BASE_PROTO_UDP;
+
+  dbus_g_type_struct_get (transport,
+                          1, &address,
+                          3, &proto,
+                          G_MAXUINT);
+
+  g_assert (address != NULL);
+
+  if (proto != TP_MEDIA_STREAM_BASE_PROTO_UDP)
+    result = -1;
+  /* XXX: this will not work properly when IPv6 support comes */
+  else if (priv->local_ip_address != NULL
+      && strcmp (address, priv->local_ip_address) == 0)
+    result = 1;
+
+  g_free (address);
+
+  return result;
+}
 
 static void priv_session_media_state (SIPMediaSession *session, gboolean playing)
 {
@@ -1166,6 +1215,7 @@ priv_local_media_changed (SIPMediaSession *session)
       break;
     case SIP_MEDIA_SESSION_STATE_INVITE_SENT:
     case SIP_MEDIA_SESSION_STATE_REINVITE_SENT:
+    case SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED:
       /* Cannot send another offer right now */
       priv->pending_offer = TRUE;
       break;
@@ -1239,7 +1289,6 @@ priv_update_remote_media (SIPMediaSession *session, gboolean authoritative)
         }
 
       /* There have been problems with the stream update, kill the stream */
-      /* XXX: fast and furious, not tested */
       sip_media_stream_close (stream);
     }
 
@@ -1456,12 +1505,9 @@ priv_request_response_step (SIPMediaSession *session)
           priv_session_invite (session, FALSE);
         }
       break;
-    case SIP_MEDIA_SESSION_STATE_INVITE_SENT:
-      if (!priv->accepted)
-        break;
-      /* if accepted, fall through to the next case */
-    case SIP_MEDIA_SESSION_STATE_REINVITE_SENT:
-      if (!priv_is_codec_intersect_pending (session))
+    case SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED:
+      if (priv->accepted
+          && !priv_is_codec_intersect_pending (session))
         {
           g_assert (priv->local_non_ready == 0);
           g_object_set (session,
@@ -1561,15 +1607,11 @@ static void priv_stream_supported_codecs_cb (SIPMediaStream *stream,
       /* This remote media description got no codec intersection. */
       switch (priv->state)
         {
-        case SIP_MEDIA_SESSION_STATE_INVITE_SENT:
+        case SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED:
         case SIP_MEDIA_SESSION_STATE_INVITE_RECEIVED:
           DEBUG("no codec intersection, closing the stream");
           sip_media_stream_close (stream);
           break;
-        case SIP_MEDIA_SESSION_STATE_REINVITE_SENT:
-          g_warning ("re-INVITE got a response with a non-intersecting codec set");
-          sip_media_stream_close (stream);
-          break;
         case SIP_MEDIA_SESSION_STATE_REINVITE_RECEIVED:
           /* In this case, we have the stream negotiated already,
            * and we don't want to close it just because the remote party
