@@ -311,6 +311,7 @@ static void
 sip_media_stream_class_init (SIPMediaStreamClass *sip_media_stream_class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (sip_media_stream_class);
+  GType stream_type = G_OBJECT_CLASS_TYPE (sip_media_stream_class);
   GParamSpec *param_spec;
 
   g_type_class_add_private (sip_media_stream_class, sizeof (SIPMediaStreamPrivate));
@@ -365,9 +366,6 @@ sip_media_stream_class_init (SIPMediaStreamClass *sip_media_stream_class)
                                   G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_MEDIA_TYPE, param_spec);
 
-  /* We don't change the following two as individual properties
-   * after construction, use sip_media_stream_set_direction() */
-
   param_spec = g_param_spec_uint ("state", "Connection state",
                                   "Connection state of the media stream",
                                   TP_MEDIA_STREAM_STATE_DISCONNECTED,
@@ -380,6 +378,9 @@ sip_media_stream_class_init (SIPMediaStreamClass *sip_media_stream_class)
                                    PROP_STATE,
                                    param_spec);
 
+  /* We don't change the following two as individual properties
+   * after construction, use sip_media_stream_set_direction() */
+
   param_spec = g_param_spec_uint ("direction", "Stream direction",
                                   "A value indicating the current "
                                         "direction of the stream",
@@ -410,7 +411,7 @@ sip_media_stream_class_init (SIPMediaStreamClass *sip_media_stream_class)
   /* signals not exported by DBus interface */
   signals[SIG_READY] =
     g_signal_new ("ready",
-                  G_OBJECT_CLASS_TYPE (sip_media_stream_class),
+                  stream_type,
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
@@ -419,7 +420,7 @@ sip_media_stream_class_init (SIPMediaStreamClass *sip_media_stream_class)
 
   signals[SIG_SUPPORTED_CODECS] =
     g_signal_new ("supported-codecs",
-                  G_OBJECT_CLASS_TYPE (sip_media_stream_class),
+                  stream_type,
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
@@ -428,7 +429,7 @@ sip_media_stream_class_init (SIPMediaStreamClass *sip_media_stream_class)
 
   signals[SIG_STATE_CHANGED] =
     g_signal_new ("state-changed",
-                  G_OBJECT_CLASS_TYPE (sip_media_stream_class),
+                  stream_type,
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
@@ -437,7 +438,7 @@ sip_media_stream_class_init (SIPMediaStreamClass *sip_media_stream_class)
 
   signals[SIG_DIRECTION_CHANGED] =
     g_signal_new ("direction-changed",
-                  G_OBJECT_CLASS_TYPE (sip_media_stream_class),
+                  stream_type,
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
@@ -549,9 +550,6 @@ sip_media_stream_native_candidates_prepared (TpSvcMediaStreamHandler *iface,
 {
   /* purpose: "Informs the connection manager that all possible native candisates
    *          have been discovered for the moment." 
-   *
-   * note: only emitted by the stream-engine when built without
-   *       libjingle (tested with s-e 0.3.11, 2006/Dec)
    */
 
   SIPMediaStream *obj = SIP_MEDIA_STREAM (iface);
@@ -559,14 +557,16 @@ sip_media_stream_native_candidates_prepared (TpSvcMediaStreamHandler *iface,
 
   g_assert (SIP_IS_MEDIA_STREAM (obj));
   priv = SIP_MEDIA_STREAM_GET_PRIVATE (obj);
-  
-  priv->native_cands_prepared = TRUE;
 
-  push_active_candidate_pair (obj);
+  DEBUG("enter");
+
+  priv->native_cands_prepared = TRUE;
 
   if (priv->native_codecs_prepared)
     priv_generate_sdp (obj);
 
+  push_active_candidate_pair (obj);
+
   tp_svc_media_stream_handler_return_from_native_candidates_prepared (context);
 }
 
@@ -620,25 +620,14 @@ sip_media_stream_new_native_candidate (TpSvcMediaStreamHandler *iface,
                                        const GPtrArray *transports,
                                        DBusGMethodInvocation *context)
 {
-  /* purpose: "Inform this MediaStreamHandler that a new native transport candidate
-   *
-   * - decide whether it's time generate an offer/answer (based on gathered
-   *   candidates); this should be done after candidates_prepared(),
-   *   but current stream-engine never emits this
-   * - create SDP segment for this stream (the m-line and associated
-   *   c-line and attributes)
-   * - mark that we've created SDP (so that additional new candidates 
-   *   can be processed correced 
-   * - emit 'Ready' when ready to send offer/answer
-   */
-
   SIPMediaStream *obj = SIP_MEDIA_STREAM (iface);
   SIPMediaStreamPrivate *priv;
   GPtrArray *candidates;
   GValue candidate = { 0, };
+  GValue transport = { 0, };
+  gint tr_goodness;
 
   g_assert (SIP_IS_MEDIA_STREAM (obj));
-
   priv = SIP_MEDIA_STREAM_GET_PRIVATE (obj);
 
   if (priv->stream_sdp != NULL)
@@ -648,11 +637,29 @@ sip_media_stream_new_native_candidate (TpSvcMediaStreamHandler *iface,
       return;
     }
 
-  g_free (priv->native_candidate_id);
-  priv->native_candidate_id = g_strdup (candidate_id);
+  g_return_if_fail (transports->len >= 1);
+
+  /* Rate the preferability of the address */
+  g_value_init (&transport, sip_tp_transport_struct_type ());
+  g_value_set_static_boxed (&transport, g_ptr_array_index (transports, 0));
+  tr_goodness = sip_media_session_rate_native_transport (priv->session,
+                                                         &transport);
 
   candidates = g_value_get_boxed (&priv->native_candidates);
 
+  if (tr_goodness > 0)
+    {
+      DEBUG("native candidate '%s' is rated as preferable", candidate_id);
+      g_free (priv->native_candidate_id);
+      priv->native_candidate_id = g_strdup (candidate_id);
+
+      /* Drop the candidates received previously */
+      g_value_reset (&priv->native_candidates);
+      candidates = dbus_g_type_specialized_construct (
+                                sip_tp_candidate_list_type ());
+      g_value_take_boxed (&priv->native_candidates, candidates);
+    }
+
   g_value_init (&candidate, sip_tp_candidate_struct_type ());
   g_value_take_boxed (&candidate,
       dbus_g_type_specialized_construct (sip_tp_candidate_struct_type ()));
@@ -666,9 +673,6 @@ sip_media_stream_new_native_candidate (TpSvcMediaStreamHandler *iface,
 
   SESSION_DEBUG(priv->session, "put native candidate '%s' from stream-engine into cache", candidate_id);
 
-  if (priv->native_codecs_prepared)
-    priv_generate_sdp (obj);
-
   tp_svc_media_stream_handler_return_from_new_native_candidate (context);
 }
 
@@ -1455,14 +1459,18 @@ priv_update_local_sdp(SIPMediaStream *stream)
   g_value_init (&transport, sip_tp_transport_struct_type ());
   g_value_init (&codec, sip_tp_codec_struct_type ());
 
-  /* Find the last acceptable candidate */ 
+  /* Find the preferred candidate, if defined,
+   * else the last acceptable candidate */
 
   for (i = candidates->len - 1; i >= 0; --i)
     {
       GValueArray *candidate;
+      const gchar *candidate_id;
       const GPtrArray *ca_tports;
 
       candidate = g_ptr_array_index (candidates, i);
+      candidate_id =
+                g_value_get_string (g_value_array_get_nth (candidate, 0));
       ca_tports = g_value_get_boxed (g_value_array_get_nth (candidate, 1));
 
       g_return_if_fail (ca_tports->len >= 1);
@@ -1482,8 +1490,17 @@ priv_update_local_sdp(SIPMediaStream *stream)
                               /* 9, &tr_pass, */
                               G_MAXUINT);
 
-      if (tr_proto == TP_MEDIA_STREAM_BASE_PROTO_UDP)
-        break;
+      if (priv->native_candidate_id != NULL)
+        {
+          if (!strcmp (candidate_id, priv->native_candidate_id))
+            break;
+        }
+      else if (tr_proto == TP_MEDIA_STREAM_BASE_PROTO_UDP)
+        {
+          g_free (priv->native_candidate_id);
+          priv->native_candidate_id = g_strdup (candidate_id);
+          break;
+        }
     }
   g_return_if_fail (i >= 0);
   g_return_if_fail (tr_addr != NULL);
@@ -1571,7 +1588,9 @@ priv_update_local_sdp(SIPMediaStream *stream)
   g_free(priv->stream_sdp);
   priv->stream_sdp = g_strconcat(mline->str, "\r\n",
                                  cline,
+#if 0   /* Some Cisco GWs do not like these */
                                  "b=RS:0\r\nb=RR:0\r\n",
+#endif
                                  alines->str,
                                  NULL);
 
